--==============================================================================
--
-- Function   : Simplex G-LINK S-LINK LDC
--              PCB#  EP 680-1110-850-A
--
-- Description: G-LINK-S-LINK LDC
--              Using 20 bit G-LINK mode only. 
--              Bits 19-16 are the odd parity of bits 15-12, 11-8, 7-4, 3-0. 
--              Words are mx-ed in 32 bit S-LINK mode using the FLAG:
--              the high (31-16) half word is received first with high Flag,
--              the low (15-0) half word is received second with low flag.
--              Selftest mode is implemented: A high value on bit 17 of a 
--              Control word start test mode, the first control word with
--              0 on bit 17 is not written to LD, it terminates Test Mode.  
--                                             
-- Control word bit allocation:
-- ______________________________________________________________________
-- | 19-18     | 17          | 16              | 15-4      | 3-0        |
-- |For G-Link | Test C.Word | Parity. of 15-4 | Ctrl Word | For S-Link |
--_______________________________________________________________________
--             
-- File name  : ldc.vhd
-- Created on : 01.July.1999
-- Developer  : Zoltan Meggyesi
-- Notes      : - Chip, Pinout and Logic options in ldc.acf file
--		        - File made for EPC1441PC8 PROM
--              - File for PCB version .850-A
--
--==============================================================================
-- 
-- Jumpers: TP(n) signal is '0' when the TP(n) jumper is plugged (on) 
--          TP3 off: Locking Methode "2" - on incoming data stream
--              on : Locking Methode "3" - with reference oscillator 
--          TP1, TP2, TP4, TP5: Reserved
--
-------------------------------------------------------------------------------|
-- Version | Mod.Date | Change made                                 | checksum |
-------------------------------------------------------------------------------|
--  0.1    | 05-07-99 | First Version                               |    -     | 
--  0.1    | 07-07-99 | Simulated Version                           |    -     |
--  0.2    | 21-07-99 | Small bugs corrected                        | 01827bb  |
--  0.2    | 22-07-99 | Link down bug corrected                     |    -     |
--  0.2    | 05-08-99 | Ctrl Bit(16) selects low/high halfword      | 0180619  |
--  0.2    | 06-08-99 | Error latch is disabled during link init.   | 01828ec  |
------------------- ---------------------------------------------------------- |

LIBRARY IEEE;
USE     IEEE.STD_LOGIC_1164.ALL;
USE     IEEE.STD_LOGIC_ARITH.ALL;
USE     IEEE.STD_LOGIC_UNSIGNED.ALL; -- Maxplus needs this, 
                                     -- comment out for leapfrog

--==============================================================================
ENTITY LDC IS
--==============================================================================

  port(
--------------------------------------------------------------------------------
--      INPUTS
--------------------------------------------------------------------------------
        QCLK            : in    std_logic;  -- 40 Mhz from Quartz
        TP              : in    std_logic_vector(5 downto 1); -- Jumpers
        
        --AD_OTI          : in    std_logic;  -- Not used
        --AD_DIO          : inout std_logic;  -- Not used

        HP_STRBOUT      : in    std_logic;  -- RX clock from HP
        HP_LINKRDY_N    : in    std_logic;  -- Low when link is active    
        HP_CAV_N        : in    std_logic;  -- G control word
        HP_DAV_N        : in    std_logic;  -- G data word
        HP_ERROR        : in    std_logic;  -- Error flag
        HP_FF           : in    std_logic;  -- Fill Frame
        HP_FLAG         : in    std_logic;
        HP_STAT1        : in    std_logic;  -- HP State Machine Output
        HP_STAT0        : in    std_logic;
   
        HP_D            : in    std_logic_vector(19 downto 0);

        URL             : in    std_logic_vector(3 downto 0);
        URESET_N        : in    std_logic;
        UXOFF_N         : in    std_logic;
        UTDO_N          : in    std_logic;
        UDW             : in    std_logic_vector(1 downto 0);

        fr_rxsigdet     : in    std_logic;
        fr_rxsys        : in    std_logic_vector(1 downto 0);

--------------------------------------------------------------------------------
--      OUTPUTS
--------------------------------------------------------------------------------
        --AD_CONVST_N     : out   std_logic;  -- Not used
        --AD_SCLK         : out   std_logic;  -- Not used
        --AD_RDWR         : out   std_logic;  -- Not used
		
        HP_EQEN         : out   std_logic; -- Cable equalization
        HP_FLAGSEL      : out   std_logic; -- Flag Mode Select
        HP_LOOPEN       : out   std_logic; -- Loopback control
        HP_M20SEL       : out   std_logic; -- 20/16 bit mode
        HP_SMRST1_N     : out   std_logic; -- State m. reset
        HP_SMRST0_N     : out   std_logic; -- State m. reset
        HP_TCLK         : out   std_logic; -- External test CLK
        HP_TCLKsel      : out   std_logic; -- Enable TCLK Input
        HP_DIV          : out   std_logic_vector(1 downto 0); -- Baud rate

        LD              : out   std_logic_vector(31 downto 0);
        LCLK            : out   std_logic; -- S-Link Signals 
        LCTRL_N         : out   std_logic;
        LWEN_N          : out   std_logic;
        LDOWN_N         : out   std_logic;
        LDERR_N         : out   std_logic;

        G               : out   std_logic_vector(25 downto 0); 
          -- Unused Pins are connected to ground to improve gnd-bounce
        CK40TTL         : out   std_logic; 
        TESTLED_N       : out   std_logic; -- Leds, lights when 0
        DERRLED_N       : out   std_logic;
        LUPLED_N        : out   std_logic;
        FLOWCTLED_N     : out   std_logic);

end LDC;

--==============================================================================
ARCHITECTURE behavioural OF LDC IS
--==============================================================================

component shift33      -- 33 bit shiftreg. for selftest pattern generation
    port(Data: in std_logic_vector(32 downto 0);
         Enable, Shiften, Shiftin, Clock: in std_logic;
	 Q: out std_logic_vector(32 downto 0));
end component;
  
TYPE   init_state  is (DOWN, UP, RES1, RES2, RES3,  LERR);
TYPE   reset_state is (OK, RSTNOW, WAITST);
-- attribute TYPE_ENCODING_STYLE of init_state:type is ONEHOT;
-- Note: the Altera settings such as coding style are in the file ldc.acf

signal curr_reset, next_reset: reset_state;                   -- Reset SM
signal curr_init, next_init  : init_state;                    -- Link init SM
signal inreg                 : std_logic_vector(19 downto 0); -- Input reg.(.q)
signal cnt                   : std_logic_vector(3 downto 0); 
signal filt                  : std_logic_vector(10 downto 0); -- Filter
signal ld_ad, LDr, pipeline, pipe3           : std_logic_vector(31 downto 0);
signal errled_n, linkrdy_n            : std_logic;
signal incav_n, indav_n, inerr, LWENr_N, lup: std_logic;
signal inflag, sys_res, hp_up, lwen_n_ad: std_logic;
signal testpattern, tdata: std_logic_vector(32 downto 0);
signal tloadena, tshiften, tshiftin, utestreg   : std_logic;
signal err, err2, err3_n, perr, perrh, perrl, t_error, elatch, testbit: std_logic;
signal longwmode, wen2_n, ctrl2_n, wen3_n, ctrl3_n, tout_n: std_logic;

begin

--------------------------------------------------------------------------------
-- Non-registered OUTPUTS ------------------------------------------------------

HP_EQEN    <= '0';                             -- Cable equalization disabled
HP_FLAGSEL <= UDW(1) nor UDW(0);               -- Use FLAG in 32 bit S-LINK mode
HP_LOOPEN  <= (not HP_STAT1) and (not TP(3));  -- Simplex Method 2 if TP3='1' (off),
                                               -- S.M 3 if TP3='0' (on)  
HP_M20SEL  <= '1';                             -- 20 bit mode is used
HP_TCLK    <= '0';                             -- Not used
HP_TCLKsel <= '0';                             -- Test Clock input Disabled
HP_DIV     <= "00";                            -- 700-1500 MBaud/sec

LCLK       <= not HP_STRBOUT;                  -- LCLK isInverted for better Tsu/Th 

-- "Besides placing switching pins next to a ground pin, you can create a 
--  programmable ground by creating an output pin in your design that drives
--  only ground. By connecting this output pin to ground on the board, the 
--  device ground will have another connection to the board ground, which helps
-- reduce ground bounce."(page 532 in 1998 ALTERA data book)
G          <=  (others => '0');                -- gnd outputs against gnd-bounce

--------------------------------------------------------------------------------
-- EDGE SENSITIVE RESET: Generates a pulse on negative URESET_N edge -----------

reset_reg :process(HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT ='1') then
      curr_reset <= next_reset;
  end if;
end process reset_reg;

reset_machine :process(curr_reset, URESET_N) begin
   case curr_reset is
      when OK =>          -- No reset, URESET_N is high
         if (URESET_N ='0') then next_reset <= RSTNOW;
         else next_reset <= OK;
         end if;

      when RSTNOW =>      -- Reset pulse 
         next_reset <= WAITST;
         
      when WAITST =>     -- Wait while URESET_N is low; 
         if (URESET_N ='1') then next_reset <= OK;
         else next_reset <= WAITST;
         end if;

      when others => next_reset <= WAITST;
   end case;
end process reset_machine;

reset_out: process (curr_reset) begin
  case curr_reset is
      when RSTNOW => sys_res <= '1'; -- Active high system reset
      when others => sys_res <= '0';
  end case;
end process reset_out;

--------------------------------------------------------------------------------
-- "Filtered" HPREADY (hpdown signal) ------------------------------------------
-- This signal has cca. 100ns wide negative pulses during 
-- synchronisation. Sync. with methode "3" @40mhz takes cca. 25ms
-- hp_up goes down if HP_LINKRDY_N goes up (G-Link Down)
-- hp_up goes up if HP_LINKRDY_N is low for >25us

filter:process(HP_STRBOUT) begin   
    if (HP_STRBOUT'event and HP_STRBOUT='1') then
      if (HP_LINKRDY_N = '1') then filt <= (others => '0');
      elsif (filt(10)='1') then filt <= filt;  -- stop counting if MSB=1 
      else filt <= filt + '1';                 -- filt(3) for sim.
      end if;
    end if;  
end process filter;

hp_up <= filt(10); -- change to filt(3) for simulation!
  
--------------------------------------------------------------------------------
-- LINK INIT STATE MACHINE -----------------------------------------------------

init_reg :process(HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
  curr_init <= next_init;
end if;
end process init_reg;


init_machine: process (curr_init, sys_res) begin
 
case curr_init is 
    when DOWN =>               -- Wait for LINKRDY; Power-On state
        if (hp_up = '1') then
          next_init <= UP;     -- Link UP if LINKRDY
        elsif (sys_res = '1') then 
          next_init <= RES1;   -- HP reset possible in DOWN state
        else next_init <= DOWN;
        end if;

    when UP =>                 -- when LINK UP
        if (sys_res = '1') then next_init <= RES1;
          elsif (hp_up = '0') then next_init <= LERR; 
          else next_init <= UP;
        end if;

    when RES1 =>                -- HP Reset Pulse Here!
        next_init <= RES2;      -- As in S-LINK spec, 
   
    when RES2 =>                -- LDOWN Period for 4 LCLK 
         if (hp_up = '0') then
           next_init <= RES3;   -- Wait for HP-Down
           else NEXT_INIT <= RES2;
         end if;        
    
    when RES3 => 
         next_init <= DOWN;  
   
    when LERR =>               -- Link error, quit with URESET#
        case sys_res is        -- According to S-LINK spec.
          when '1'    => next_init <= RES1;  -- Reset HP
          when others => next_init <= LERR;
        end case;
   
    when others =>             -- Same as LERR (Link Error)
        case sys_res is
          when '1'    => next_init <= RES1;
          when others => next_init <= LERR;
	    end case;
end case;
end process init_machine;

init_out: process (HP_STRBOUT) begin
if (HP_STRBOUT'event and HP_STRBOUT='1') then
   case curr_init is
     when UP =>     LDOWN_N     <= not utestreg; -- Link is active in UP state
                    LUPLED_N    <= utestreg;     -- But DOWN in selftest mode
                    HP_SMRST1_N <= '1';
                    lup         <= '1';
     when RES1 =>   LDOWN_N     <= '0';          -- LDOWN is '0' in Test Mode
                    LUPLED_N    <= '1';
                    HP_SMRST1_N <= '0';          -- Reset HP Here!
                    lup         <= '0';
     when others => LDOWN_N     <= '0';
                    LUPLED_N    <= '1';
                    HP_SMRST1_N <= '1';
                    lup         <= '0';
     end case;
end if; 
end process init_out;

HP_SMRST0_N <= '1';                     -- Not used, reset only with
                                        -- HP_SMRST1_N

refclock: process (curr_init, TP(3)) begin
case TP(3) is
  when '1' => CK40TTL <= '0';           -- TP3 off, Lock Methode "2" 
  when others =>                        -- TP3 on, Lock Methode "3"
    case curr_init is
      when UP =>     CK40TTL   <= '0';  -- Disabled when UP for less PCB noise
	  when others => CK40TTL   <= QCLK; -- Send Reference Clock for G-Link
    end case;
end case;
end process refclock;

--------------------------------------------------------------------------------
-- DATA ERROR and FLOWCTRL LEDS ------------------------------------------------

e_led: process (HP_STRBOUT) begin 
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
    if (curr_init /= UP) then errled_n <= '1';  -- Goes off on URESET# or LDOWN#
      elsif (err ='1') then errled_n <= '0';    -- Illuminates on error signal
      else errled_n <= errled_n;
    end if; 
  end if;
end process e_led;
DERRLED_N <= errled_n;

f_led: process (HP_STRBOUT) begin  
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
    if (UXOFF_N = '0') then FLOWCTLED_N <= '0';   -- Goes on on UXOFF_N#
    else FLOWCTLED_N <= '1';
    end if;
  end if;
end process f_led;

--------------------------------------------------------------------------------
-----INPUT REGISTER (1. pipeline stage)-----------------------------------------

inputreg: process (HP_STRBOUT) begin
    if (HP_STRBOUT'event and HP_STRBOUT='1') then

      inreg                <= HP_D;         -- (19 downto 0)
      indav_n              <= HP_DAV_N; 
      incav_n              <= HP_CAV_N;
      inerr                <= HP_ERROR;
      inflag               <= HP_FLAG;
      linkrdy_n            <= HP_LINKRDY_N;
     
    end if;
end process inputreg;

--------------------------------------------------------------------------------
-----PARITY CHECK (1. pipeline stage)-----------------------------------------

parchk: process(inreg) begin
 
  case incav_n is
    when '1' =>       -- Data word

    perr <=(inreg(15) xor inreg(14) xor inreg(13) xor inreg(12) xor inreg(19)) or 
           (inreg(11) xor inreg(10) xor inreg(9)  xor inreg(8)  xor inreg(18)) or 
           (inreg(7)  xor inreg(6)  xor inreg(5)  xor inreg(4)  xor inreg(17)) or
           (inreg(3)  xor inreg(2)  xor inreg(1)  xor inreg(0)  xor inreg(16));
    
    when others =>       -- Control word
    
    perr <= '0';         -- No Parity check, ireg(16) is used for demultiplexing 
  end case;

end process parchk;

--------------------------------------------------------------------------------
-----32 BIT MODE DEMUX (2. pipeline stage) -------------------------------------

longwmode <= UDW(0) nor UDW(1);  -- '1' in 32 bit mode

dmx: process (HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT='1') then

   if (( longwmode and ( (incav_n and inflag) or (not(incav_n) and inreg(16)) ) )='1')   
   then     -- 32 bit mode, high halfword: flag on data, bit(3) on ctrl
      pipeline(31 downto 16) <= inreg(15 downto 0);
      perrh <= perr;
      perrl <= '0';
      wen2_n <= '1'; -- High halfword tranmitted first, do not write it
      ctrl2_n <= '1';
    else
      pipeline(15 downto 0) <= inreg(15 downto 0); -- low halfword
      perrl  <= perr;
      perrh  <= perrh and longwmode;  -- '0' in 16 bit mode; keep otherwise
      wen2_n <= (indav_n and incav_n) or -- Write data/control here!
                (testbit and not(incav_n) and not(inreg(17))); 
       -- Do not write control word after Tmode (it terminates tmode). 
      ctrl2_n <= incav_n; 
  end if;                                           
  
  err2 <= inerr;

 end if;
end process dmx;

--------------------------------------------------------------------------------
-----3. PIPELINE ---------------------------------------------------------------
pipe: process (HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
    
    pipe3(31 downto 4) <= pipeline (31 downto 4); 
    
    case ctrl2_n is
      when '1' =>
        pipe3(3 downto 0)   <= pipeline(3 downto 0);  -- Data word
      when others =>                   
        pipe3(3) <= '0';                    -- Reserved bit
        pipe3(2) <= '0';                    -- Reserved bit
        pipe3(1) <= perrh or perrl or err2; -- Error in ctrl word
        pipe3(0) <= elatch;                 -- Error in previous data block
    end case;

  err3_n  <= (not(perrh or perrl or err2)) or wen2_n;   -- not(t_error or err2);
  err     <= (not err3_n) or t_error;       -- this illuminates the Error LED
  wen3_n  <= wen2_n or (utestreg and tout_n) or (not lup);
          -- Do not writetest data when tout is high or when the link is down
  ctrl3_n <= ctrl2_n or (utestreg and tout_n);
  
  end if;
end process pipe;

--------------------------------------------------------------------------------
-----TEST MODE LATCH -----------------------------------------------------------

testmode: process (HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
    testbit <= utestreg;  
-- Delays utestereg, in order to write the last test data word in 32 bit mode

    if ((incav_n='0') and (inreg(17)='1') and (lup='1')) then 
                                        -- Test Control Word and Link up 
    utestreg <= '1';
    elsif ( (sys_res='1') or 
            ((incav_n='0') and (inreg(17)='0')) ) then
                                             -- Ctr word, it won't be sent to user
    utestreg <= '0';                         -- Quits on 1st. control wd after tmode
    else utestreg <= utestreg;              
    end if;
  end if;
end process testmode;

TESTLED_N  <= not utestreg;

--------------------------------------------------------------------------------
-----UTDO# LATCH ---------------------------------------------------------------

utdo: process (HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
    case curr_init is
      when UP => tout_n <= tout_n;      -- Keep value
      when others => tout_n <= UTDO_N;   -- Update on reset
    end case;
  end if;
end process utdo;

--------------------------------------------------------------------------------
-----TEST DATA CHECKER ---------------------------------------------------------
 
tcheck: shift33
        port map (testpattern, tloadena, tshiften, tshiftin, HP_STRBOUT, tdata);

testpattern(0) <= '1';
testpattern(32 downto 1) <= "00000000000000000000000000000000"; -- Load shiftreg in normal mode
tloadena                 <= not utestreg;  

tshiftin  <= (tdata(16) and not(longwmode)) or (tdata(32) and longwmode);
tshiften  <= utestreg and (not (wen2_n and ctrl2_n)); -- shift enable

compar: process (HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT='1') then

    case longwmode is 
      when '0' =>
       if ( (pipeline(15 downto 0) /= tdata(16 downto 1)) 
        and (utestreg='1') and ((wen2_n and ctrl2_n)='0') )  then
        t_error <= '1';
        else t_error <= '0';
        end if;
      when others =>
       if ( (pipeline /= tdata(32 downto 1))      -- 31 bit mode
        and (utestreg='1') and ((wen2_n and ctrl2_n)='0') ) then
        t_error <= '1';
        else t_error <= '0';
        end if;

    end case;
  end if;
end process compar;

-- ERROR LATCH -----------------------------------------------------------------

errlatch: process (HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
    if ((curr_init /= UP) or (ctrl3_n = '0')) then  -- Disable during initialisation
      elatch <= '0';                                -- Clear on Ctrl Word
    elsif ( (not(err3_n) or t_error) = '1') then -- Error in data word
      elatch <= '1';
    else elatch <= elatch;  -- Keep value
    end if;
  end if;
end process errlatch;

--------------------------------------------------------------------------------
-- OUTPUT REGISTER -------------------------------------------------------------

outputreg: process (HP_STRBOUT) begin
  if (HP_STRBOUT'event and HP_STRBOUT='1') then
       
       LD      <= pipe3;      
       LDERR_N <= err3_n and not(t_error) and (ctrl3_n or not(elatch));
         -- Active low or err3_n and -- on selftest pattern error
       LWEN_N  <= wen3_n;   
       LCTRL_N <= ctrl3_n;

  end if;
end process outputreg;
  
end behavioural;

--==============================================================================
--     END OF FILE
--==============================================================================
